==> pam-0.3.tar.gz: Linux-PAM-0.3/libpam/pam_item.c <==
/* pam_item.c */

/*
 * $Id: pam_item.c,v 1.2 1996/03/16 21:53:31 morgan Exp $
 *
 * $Log: pam_item.c,v $
 * Revision 1.2  1996/03/16 21:53:31  morgan
 * deleted comments about how to restrict the access to AUTHTOK
 * things... this is dealt with by having moved the tokens to
 * pam_modules.h
 * Overwrite passwords with zeros before freeing their memory
 *
 */

#include <stdlib.h>
#include <string.h>
#include <syslog.h>

#include "pam_private.h"

#define RESET(X, Y) \
if (X) { \
    free(X); \
} \
(X) = (Y) ? strdup(Y) : NULL

/* functions */

int pam_set_item (
    pam_handle_t *pamh,
    int item_type,
    const void *item)
{
    int retval;

    IF_NO_PAMH("pam_set_item",pamh,PAM_SYSTEM_ERR);
    
    retval = PAM_SUCCESS;

    switch (item_type) {
    case PAM_SERVICE:
	/* Setting handlers_loaded to 0 will cause the handlers
	 * to be reloaded on the next call to a service module.
	 */
	pamh->handlers.handlers_loaded = 0;
	RESET(pamh->service_name, item);
	break;
    case PAM_USER:
	RESET(pamh->user, item);
	break;
    case PAM_USER_PROMPT:
	RESET(pamh->prompt, item);
	break;
    case PAM_TTY:
	RESET(pamh->tty, item);
	break;
    case PAM_RUSER:
	RESET(pamh->ruser, item);
	break;
    case PAM_RHOST:
	RESET(pamh->rhost, item);
	break;
    case PAM_AUTHTOK:
	 /* XXX - The man page says this is only supposed to
	  * be available to the module providers.  Not sure
	  * how to do that...
	  */
	 _pam_overwrite(pamh->authtok);
	 RESET(pamh->authtok, item);
	break;
    case PAM_OLDAUTHTOK:
	 /* XXX - This too. */
	 _pam_overwrite(pamh->oldauthtok);
	 RESET(pamh->oldauthtok, item);
	 break;
    case PAM_CONV:              /* want to change the conversation function */
	if (item == NULL) {
	    _pam_log_error("pam_set_item: attempt to set conv() to NULL");
	    retval = PAM_PERM_DENIED;
	} else {
	    struct pam_conv *tconv;
	    
	    if ((tconv=
		 (struct pam_conv *) malloc(sizeof(struct pam_conv))
		) == NULL) {
		_pam_log_error("pam_set_item: malloc failed for pam_conv");
		retval = PAM_BUF_ERR;
	    } else {
		DROP(pamh->pam_conversation);
		memcpy(pamh->pam_conversation=tconv
		       , item, sizeof(struct pam_conv));
	    }
	}
        break;
    default:
	retval = PAM_BAD_ITEM;
    }

    return (retval);
}

int pam_get_item (
    const pam_handle_t *pamh,
    int item_type,
    const void **item)
{
    IF_NO_PAMH("pam_set_item",pamh,PAM_SYSTEM_ERR);
    
    switch (item_type) {
    case PAM_SERVICE:
	*item = pamh->service_name;
	break;
    case PAM_USER:
	*item = pamh->user;
	break;
    case PAM_USER_PROMPT:
	*item = pamh->prompt;
	break;
    case PAM_TTY:
	*item = pamh->tty;
	break;
    case PAM_RUSER:
	*item = pamh->ruser;
	break;
    case PAM_RHOST:
	*item = pamh->rhost;
	break;
    case PAM_AUTHTOK:
	*item = pamh->authtok;
	break;
    case PAM_OLDAUTHTOK:
	*item = pamh->oldauthtok;
	break;
    case PAM_CONV:
	*item = pamh->pam_conversation;
	break;
    default:
	/* XXX - I made this up */
	return PAM_BAD_ITEM;
    }
  
    return PAM_SUCCESS;
}

/* added by AGM 1996/3/2 */

int pam_get_user(pam_handle_t *pamh, char **user, const char *prompt)
{
    const char *use_prompt;
    int retval;
    struct pam_message msg,*pmsg;
    struct pam_response *resp;

    IF_NO_PAMH("pam_get_user",pamh,PAM_SYSTEM_ERR);

    DROP(pamh->tuser);   /* first drop previous temp username */

    if (pamh->pam_conversation == NULL) {
	_pam_log_error("pam_get_user: no conv element in pamh");
	return PAM_SERVICE_ERR;
    }

    if (user == NULL) {  /* ensure the the module has suplied a destination */
	_pam_log_error("pam_get_user: no where to record username");
	return PAM_PERM_DENIED;
    } else
	*user = NULL;
    
    if (pamh->user) {    /* have one so return it */
	*user = pamh->user;
	return PAM_SUCCESS;
    }

    /* will need a prompt */

    use_prompt = prompt;
    if (use_prompt == NULL) {
	use_prompt = pamh->prompt;
	if (use_prompt == NULL) {
	    use_prompt = PAM_DEFAULT_PROMPT;
	}
    }

    /* converse with application -- prompt user for a username */

    pmsg = &msg;
    msg.msg_style = PAM_PROMPT_ECHO_ON;
    msg.msg = use_prompt;
    resp = NULL;

    retval = pamh->pam_conversation
	->conv(1, (const struct pam_message **) &pmsg
	       , &resp, pamh->pam_conversation->appdata_ptr);

    if (resp == NULL) {
	_pam_log_error("pam_get_user: no where to record username");
	return PAM_CONV_ERR;
    }

    if (retval == PAM_SUCCESS) {            /* copy the username */
	RESET(pamh->tuser, resp->resp);
	*user = pamh->tuser;

	/* note 'resp' is allocated by the application and is
           correctly free()'d here */
	DROP(resp->resp);
	DROP(resp);
    }

    return retval;        /* pass on any error from conversation */
}

==> pam-0.21.tar.gz: Linux-PAM-0.21/libpam/pam_item.c <==
/* pam_item.c */

#include <stdlib.h>
#include <string.h>
#include <syslog.h>

#include "pam_private.h"

#define RESET(X, Y) \
if (X) { \
    free(X); \
} \
(X) = (Y) ? strdup(Y) : NULL

/* functions */

int pam_set_item (
    pam_handle_t *pamh,
    int item_type,
    const void *item)
{
    int retval;

    IF_NO_PAMH("pam_set_item",pamh,PAM_SYSTEM_ERR);
    
    retval = PAM_SUCCESS;

    switch (item_type) {
    case PAM_SERVICE:
	/* Setting handlers_loaded to 0 will cause the handlers
	 * to be reloaded on the next call to a service module.
	 */
	pamh->handlers.handlers_loaded = 0;
	RESET(pamh->service_name, item);
	break;
    case PAM_USER:
	RESET(pamh->user, item);
	break;
    case PAM_USER_PROMPT:
	RESET(pamh->prompt, item);
	break;
    case PAM_TTY:
	RESET(pamh->tty, item);
	break;
    case PAM_RUSER:
	RESET(pamh->ruser, item);
	break;
    case PAM_RHOST:
	RESET(pamh->rhost, item);
	break;
    case PAM_AUTHTOK:
	/* XXX - The man page says this is only supposed to
	 * be available to the module providers.  Not sure
	 * how to do that...
	 */
	RESET(pamh->authtok, item);
	break;
    case PAM_OLDAUTHTOK:
	/* XXX - This too. */
	RESET(pamh->oldauthtok, item);
	break;
    case PAM_CONV:              /* want to change the conversation function */
	if (item == NULL) {
	    _pam_log_error("pam_set_item: attempt to set conv() to NULL");
	    retval = PAM_PERM_DENIED;
	} else {
	    struct pam_conv *tconv;
	    
	    if ((tconv=
		 (struct pam_conv *) malloc(sizeof(struct pam_conv))
		) == NULL) {
		_pam_log_error("pam_set_item: malloc failed for pam_conv");
		retval = PAM_BUF_ERR;
	    } else {
		DROP(pamh->pam_conversation);
		memcpy(pamh->pam_conversation=tconv
		       , item, sizeof(struct pam_conv));
	    }
	}
        break;
    default:
	retval = PAM_BAD_ITEM;
    }

    return (retval);
}

int pam_get_item (
    const pam_handle_t *pamh,
    int item_type,
    void **item)
{
    IF_NO_PAMH("pam_set_item",pamh,PAM_SYSTEM_ERR);
    
    switch (item_type) {
    case PAM_SERVICE:
	*item = pamh->service_name;
	break;
    case PAM_USER:
	*item = pamh->user;
	break;
    case PAM_USER_PROMPT:
	*item = pamh->prompt;
	break;
    case PAM_TTY:
	*item = pamh->tty;
	break;
    case PAM_RUSER:
	*item = pamh->ruser;
	break;
    case PAM_RHOST:
	*item = pamh->rhost;
	break;
    case PAM_AUTHTOK:
	*item = pamh->authtok;
	break;
    case PAM_OLDAUTHTOK:
	*item = pamh->oldauthtok;
	break;
    case PAM_CONV:
	*item = pamh->pam_conversation;
	break;
    default:
	/* XXX - I made this up */
	return PAM_BAD_ITEM;
    }
  
    return PAM_SUCCESS;
}

/* added by AGM 1996/3/2 */

int pam_get_user(pam_handle_t *pamh, char **user, const char *prompt)
{
    const char *use_prompt;
    int retval;
    struct pam_message msg,*pmsg;
    struct pam_response *resp;

    IF_NO_PAMH("pam_get_user",pamh,PAM_SYSTEM_ERR);

    DROP(pamh->tuser);   /* first drop previous temp username */

    if (pamh->pam_conversation == NULL) {
	_pam_log_error("pam_get_user: no conv element in pamh");
	return PAM_SERVICE_ERR;
    }

    if (user == NULL) {  /* ensure the the module has suplied a destination */
	_pam_log_error("pam_get_user: no where to record username");
	return PAM_PERM_DENIED;
    } else
	*user = NULL;
    
    if (pamh->user) {    /* have one so return it */
	*user = pamh->user;
	return PAM_SUCCESS;
    }

    /* calling application so drop any passwords! */
    
    DROP(pamh->authtok);
    DROP(pamh->oldauthtok);

    /* will need a prompt */

    use_prompt = prompt;
    if (use_prompt == NULL) {
	use_prompt = pamh->prompt;
	if (use_prompt == NULL) {
	    use_prompt = PAM_DEFAULT_PROMPT;
	}
    }

    /* converse with application -- prompt user for a username */

    pmsg = &msg;
    msg.msg_style = PAM_PROMPT_ECHO_ON;
    msg.msg = use_prompt;
    resp = NULL;

    retval = pamh->pam_conversation
	->conv(1, (const struct pam_message **) &pmsg
	       , &resp, pamh->pam_conversation->appdata_ptr);

    if (resp == NULL) {
	_pam_log_error("pam_get_user: no where to record username");
	return PAM_CONV_ERR;
    }

    if (retval == PAM_SUCCESS) {            /* copy the username */
	RESET(pamh->tuser, resp->resp);
	*user = pamh->tuser;

	/* note 'resp' is allocated by the application and is
           correctly free()'d here */
	DROP(resp->resp);
	DROP(resp);
    }

    return retval;        /* pass on any error from conversation */
}
